;+-----------------------------------------------------------------------------
;| Title                            : data-temp
;+-----------------------------------------------------------------------------
;| Funktion                         : Aktuelles Datum, Zeit und Temperatur wird auf LCD angezeigt
;| Schaltung                        : RTC, Temp und LCD
;+-----------------------------------------------------------------------------
;| Prozessor                        : ATmega8
;| Takt                             : 3,6864 MHz
;| Sprache                          : Assembler
;| Datum                            : 29.01.2011
;| Version                          : 1.0
;| Autor                            : Liedl Josef
;+-----------------------------------------------------------------------------
.include 		"m8def.inc"
.equ		F_CPU 	= 	36846400
;-------- TWI Gerteliste ---------------------------------------------------------
;			Gerte_ADR,				GerteTypID|whlbarar Adressanteil (Jumper)
.equ		RTC_ADR 	=			(0b11010000|0b00000000)
.equ		LM75_ADR	=			(0b10010000|0b00000000)
;--------------------------------------------------------------------------------------
.def 		save_sreg 	= r0		; Sicherungskopie von SREG in Interrups
.def  		iwr0    	= r1		; InterruptWorkRegister
.def  		key_ct0    	= r2		; Zhler fr Tastenabfrage
.def  		key_ct1    	= r3		; Zhler fr Tastenabfrage
.def  		key_state   = r4		; Aktueller Status von den Eingngen
.def  		key_press  	= r5		; Tasten die gerade gedrckt wurden
.def		input		= r6		; Speicher fr den Aktuellen Menabschnitt
.def		alarm_activ	= r7		; Statusregister fr Alarm Bit0 = 1 = Alarm ein
;------------------------------------------------------------------------------
begin:		rjmp	main			;  RESET External Pin, Power-on Reset, Brown-out Reset and Watchdog Reset
			reti					;  INT0 External Interrupt Request 0
			reti					;  INT1 External Interrupt Request 1
			reti					;  TIMER2 COMP Timer/Counter2 Compare Match
			rjmp	onTC2			;  TIMER2 OVF Timer/Counter2 Overflow
			reti					;  TIMER1 CAPT Timer/Counter1 Capture Event
			reti					;  TIMER1 COMPA Timer/Counter1 Compare Match A
			reti					;  TIMER1 COMPB Timer/Counter1 Compare Match B
			rjmp	onTC1			;  TIMER1 OVF Timer/Counter1 Overflow
			rjmp	onTC0			;  TIMER0 OVF Timer/Counter0 Overflow
			reti					;  SPI, STC Serial Transfer Complete
			reti					;  USART, RXC USART, Rx Complete
			reti					;  USART, UDRE USART Data Register Empty
			reti					;  USART, TXC USART, Tx Complete
			reti					;  ADC ADC Conversion Complete
			reti					;  EE_RDY EEPROM Ready
			reti					;  ANA_COMP Analog Comparator
			reti					;  TWI 2-wire Serial Interface
			reti					;  SPM_RDY Store Program Memory Ready
;------------------------------------------------------------------------------
main:		ldi		r16,high(RAMEND)	; Main program start
			out		SPH,r16			; Set Stack Pointer to top of RAM
			ldi		r16,low(RAMEND)
			out		SPL,r16
;-----------------------------------------------------------------
			rcall	twiInitMaster	; TWI initialisieren
			rcall	init_LM75		; Temp-Sensor initialisieren
			rcall	init_RTC     	; RTC initialisieren
			rcall	LCD_init		; LCD initialisieren
			rcall	LCD_clear		; LCD leeren
;----------Port init_____________________________________________
			sbi		DDRB, 5			; Ausgang Summer
			cbi		DDRB, 2			; Eingang Taster
			sbi		PORTB, 2		; PULL-UP
			cbi		DDRB, 3			; Eingang Taster
			sbi		PORTB, 3		; PULL-UP
;-----------timer init --------------------------------------
			ldi		r16, 0b00000101	; Vorteiler 1024
			out		TCCR0, r16
			clr		key_press		; key_press leeren
			;--Timer 1 = Tondauer - Melodie - Alarm
			ldi		r16, 0b00000000		; kein Vorteiler (Wird gesezt bei Alarm)
			out		TCCR1B, r16			; Timer auf Basis 1/16 s laden
			ldi		r16, dauerH			; Konstanten in alarm.s
			out		TCNT1H, r16
			ldi		r16, dauerL
			out		TCNT1L, r16
			;--Timer 2 = Frequenz - Melodie - Alarm
			ldi		r16, 0b00000000			; kein Vorteiler (Wird gesezt bei Alarm)
			out		TCCR2, r16
			;--Timer INT-Overflow -------
			ldi		r16, 0b01000101			; Interrups fr Timer 0/1/2 aktivieren
			out		TIMSK, r16
			; Ende Interruptmaskierung
			; Register fr Alarm vorbereiten
			ldi		r23, 0			; Halbwelleninfo
			ldi		r22, quite		; Frequenz
			ldi		r21, d16		; Dauer bis es losgeht
			;-- Melodie auf Anfang ---
			ldi		r28, low(melodie*2)	; Low Teil in Y
			ldi		r29, high(melodie*2)	; High Teil in Y
			;-----------register------------------------------------------
			ldi		r16, 0
			mov		alarm_activ, r16
			;--------------------------------
			; Watchdog-Timer
			wdr
			ldi		r16, 0b00011000		; Enable-Sequenz START
			out		WDTCR, r16
			ldi		r16, 0b00001101		; Watchdog ON
			out		WDTCR, r16			; Prescale 101 = 0,52 s TIMEOUT
			sei							; Interrups erlauben
;--------------------------------------------------------------------------
mainloop0:	; register ----------------------------
			ldi		r16,0
			mov		input, r16			; input (rck)setzen
mainloop:	wdr							; Watchdog-Timer zurcksetzen
			;--Tasten-------------
			cli							; Interrups sperren
 			mov 	r19, key_press   	; Tasten bergeben
			clr  	key_press   		; rcksetzen
			sei							; Interrups erlauben
			sbrc	r19,2				; wenn Taster 1 gedrckt
			rjmp	t1_main				; dann t1_main
			sbrc	r19,3				; wenn Taster 2 gedrckt
			rjmp	t2_main				; dann t2_main
			;-------------------------
mainloop1:	rcall	lcd_out					; normalte LCD Ausgabe
			ldi		r30, low(eep_alarm/2)	; Adress fr Weckzeit laden
			ldi		r31, high(eep_alarm/2)
			rcall	comp_alarm				; Weckzeit mit aktuelle Zeit vergleichen
			rjmp	mainloop				; von Vorne
;-------------------------------------------------------------------------------
; Auswahl Men
menu_selection:
			mov		r16, input				; Wert bergeben
			andi	r16, 0b00001111			; nur untere Hlfte
			ldi		r30, low(menu_m_table)	; Adresse Sprungtabelle laden
			ldi		r31, high(menu_m_table)
			add		r30, r16				; Wert addieren
			ldi		r16, 0					; Null laden
			adc		r31, r16				; evtl. bertrag addieren
			ijmp							; Springen
			;------------
m_alarm_adjust:	
			rcall	LCD_clear				; LCD leeren
			rcall	LCD_home				; LCD 1. Zeile 1. Zeichen
			ldi		r30, low(txt_alarm*2)	; Adresse fr Zeichenkette laden
			ldi		r31, high(txt_alarm*2)
			rcall	send_text				; Text ausgeben
			ldi		r16, 0x10				; Cursor auerhalb vom Sichtfeld
			rcall	LCD_goto				
			ldi		r16, 1					; Nchster Menu Abschnitt 1 laden
			mov		input, r16
			rjmp	menu_selection1			; springe zu menu_selection1
			;---------------
m_time_adjust:
			rcall	LCD_clear				; LCD leeren
			rcall	LCD_home				; LCD 1. Zeile 1. Zeichen
			ldi		r30, low(txt_date*2)	; Adresse fr Zeichenkette laden
			ldi		r31, high(txt_date*2)
			rcall	send_text				; Text ausgeben
			ldi		r16, '/'				; / laden
			rcall	LCD_data				; Senden
			ldi		r30, low(txt_time*2)	; Adresse fr Zeichenkette laden
			ldi		r31, high(txt_time*2)	
			rcall	send_text				; Text ausgeben
			ldi		r16, 0x10				; Cursor auerhalb Sichtbereich
			rcall	LCD_goto				
			ldi		r16, 2					; Nchster Menu Abschnitt 2 laden
			mov		input, r16
			rjmp	menu_selection1			; springe zu menu_selection1
			;------------------
m_back:		rcall	LCD_clear				; LCD leeren
			rcall	LCD_home				; LCD 1. Zeile 1. Zeichen
			ldi		r30, low(txt_back*2)	; Adresse fr Zeichenkette laden
			ldi		r31, high(txt_back*2)
			rcall	send_text				; Text ausgeben
			ldi		r16, 0x10				; Cursor auerhalb Sichtbereich
			rcall	LCD_goto		
			ldi		r16, 0					; Nchster Menu Abschnitt 0 laden
			mov		input, r16				
			;----------------------
menu_selection1:
			wdr								; wdr Rcksetzen
			;--Tasten-------------
			cli								; Interrups sperren
 			mov 	r19, key_press   		; Wert bergeben
			clr  	key_press   			; rcksetzten
			sei								; Interrups freigeben
			sbrc	r19,2					; wenn Taster1 gedrckt
			rjmp	t1_menu_s				
			sbrc	r19,3					; wenn Taster2 gedrckt
			rjmp	t2_menu_s
			rjmp	menu_selection1			; von Vorne
;------------------------------------------------------------------------------
time_adjust:rcall	LCD_clear				; LCD leeren
			rcall	LCD_home				; LCD 1. Zeile 1. Zeichen
			ldi		r30, low(txt_date*2)	; Adresse fr Zeichenkette laden
			ldi		r31, high(txt_date*2)	
			rcall	send_text				; Text ausgeben
			ldi		r16, 0x40				; 2. Zeile 1. Zeichen
			rcall	LCD_goto					
			rcall	sendDay					; Wochentag senden
			ldi		r16,' '					; Leerzeichen sendne
			rcall	LCD_data
			ldi		r16,' '
			rcall	LCD_data
			ldi		r16,' '
			rcall	LCD_data
			ldi		r16,' '
			rcall	LCD_data			
			rcall	sendDate				; Datum ausgeben
			ldi		r16, 0x4F				; Cursor auf Jahrauswahl
			rcall	LCD_goto
			;---------------------------
time_year:	rcall	getYear					; Jahr laden		
			mov		r17, r16				; in r17 laden
			ldi		r16, 0					; Menu Abschnitt 0
			mov		input, r16
time_year1:	ldi		r16, 0x4E				; Cursor auf Jahr
			rcall	LCD_goto
			mov		r16, r17				; in r16 laden
			rcall	sendBCD					; senden
			ldi		r16, 0x4F				; Cursor auf Jahrauswahl
			rcall	LCD_goto				
			rjmp	time_adjust1			; spring zu time_adjust1
			;--------------------------
time_month:	rcall	getMonth				; Monat laden
			mov		r17, r16				; in r17 laden
			ldi		r16, 1					; Menu Abschnitt 1
			mov		input, r16
time_month1:ldi		r16, 0x49				; Cursor auf Monat
			rcall	LCD_goto
			mov		r16, r17				; in r16 laden
			rcall	sendBCD					; senden
			ldi		r16, 0x4A				; Cursor auf Monatauswahl
			rcall	LCD_goto				
			rjmp	time_adjust1			; spring zu time_adjust1
			;-----------------------------
time_date:	rcall	getDate					; Tag laden
			mov		r17, r16				; in r17 laden
			ldi		r16, 2					; Menu Abschnitt 2
			mov		input, r16				
time_date1:	ldi		r16, 0x46				; Cursor auf Tag
			rcall	LCD_goto
			mov		r16, r17				; in r16 laden
			rcall	sendBCD					; senden
			ldi		r16, 0x47				; Cursor auf Tagauswahl
			rcall	LCD_goto
			rjmp	time_adjust1			; spring zu time_adjsut1
			;-----------------------------	
time_day:	rcall	getDay					; Wochentag laden
			mov		r17, r16				; in r17 laden
			ldi		r16, 3					; Menu Abschnitt 3
			mov		input, r16
time_day1:	ldi		r16, 0x40				; Cursor auf Wochentag
			rcall	LCD_goto			
			mov		r16, r17				; in r16 laden
			rcall	sendDay1				; senden
			ldi		r16, 0x41				; Cursor auf Wochentagauswahl
			rcall	LCD_goto				
			rjmp	time_adjust1			; spring zu time_adjust1
			;-----------------------------
time_hour:	rcall	LCD_clear				; LCD leeren
			rcall	LCD_home				; LCD 1. Zeile 1. Zeichen
			ldi		r30, low(txt_time*2)	; Adresse fr Zeichenkette laden
			ldi		r31, high(txt_time*2)
			rcall	send_text				; Text ausgeben
			ldi		r16, 0x48				; 2. Zeile 1. Zeichen
			rcall	LCD_goto			
			rcall	send_time				; Uhrzeit ausgeben
			rcall	getHour					; Stunden laden
			mov		r17, r16				; in r17 laden
			ldi		r16, 4					; Menu Abschnitt 4
			mov		input, r16
time_hour1:	ldi		r16, 0x48				; Cursor auf Stunden
			rcall	LCD_goto				
			mov		r16, r17				; in r16 laden
			rcall	sendBCD					; senden
			ldi		r16, 0x49				; Cursor auf Stundenauswahl
			rcall	LCD_goto
			rjmp	time_adjust1			; spring zu time_adjust1
			;-----------------------------
time_minute:		
			rcall	getMinute				; Minuten laden
			mov		r17, r16				; in r17 laden
			ldi		r16, 5					; Menu Abschnitt 5
			mov		input, r16
time_minute1:		
			ldi		r16, 0x4B				; Cursor auf Minuten
			rcall	LCD_goto
			mov		r16, r17				; in r16 laden
			rcall	sendBCD					; senden
			ldi		r16, 0x4C				; Cursor auf Minutenauswahl
			rcall	LCD_goto
			rjmp	time_adjust1			; spring zu time_adjust1
			;-----------------------------
time_second:	
			rcall	getSecond				; Sekunden laden
			mov		r17, r16				; in r17 laden
			ldi		r16, 6					; Menu Abschnitt 6
			mov		input, r16
time_second1:	
			ldi		r16, 0x4E				; Cursor auf Sekunden
			rcall	LCD_goto
			mov		r16, r17				; in r16 laden
			rcall	sendBCD					; senden
			ldi		r16, 0x4F				; Cursor auf Sekundenauswahl
			rcall	LCD_goto
			;-----------------------------
time_adjust1:
			wdr								; wdr Rcksetzen
			;--Tasten---------------------
			cli								; Interrups sperren
			mov		r19, key_press			; in r19 laden
			clr		key_press				; rcksetzen
			sei								; Interrups freigeben
			sbrc	r19, 2					; Wenn Taster1
			rjmp	t1_time_a
			sbrc	r19, 3					; Wenn Taster2
			rjmp	t2_time_a
			rjmp	time_adjust1			; von Vorne
;-----------------------------------------------------------------------------
alarm_adjust:
			rcall	LCD_clear				; LCD leeren
			rcall	LCD_home				; LCD 1. Zeile 1. Zeichen
			ldi		r30, low(txt_alarm*2)	; Adress fr Zeichenkette laden
			ldi		r31, high(txt_alarm*2)
			rcall	send_text				; Text ausgeben
			ldi		r16, 0x48				; LCD 2. Zeile 8. Zeichen (Stunden)
			rcall	LCD_goto
			ldi		r30, low(eep_alarm/2)	; Adress fr Weckzeit laden
			ldi		r31, high(eep_alarm/2)
			lpm		r24, Z+			; Adresse EEPROM laden
			lpm		r25, Z
			rcall	EERead			; Vergleichswert fr Stunden aus EEPROM lesen
			mov		r16, r18		; Wert in r16 verschieben
			rcall	sendBCD			; senden
			ldi		r16, ':'		; Trennzeichen
			rcall	LCD_data		; senden
			adiw	r24, 1			; Adress um 1 erhhen
			rcall	EERead			; Vergleichswert fr Minuten laden
			mov		r16, r18		; Wert in r16 verscheiben
			rcall	sendBCD			; senden
			ldi		r16, ':'		; Trennzeichen
			rcall	LCD_data		; senden
			adiw	r24, 1			; 1 erhhen
			rcall	EERead			; Wert fr Sekunden laden
			mov		r16, r18		; Wert in r16 laden
			rcall	sendBCD			; senden
			ldi		r16, 0x49		; Cursor auf Studnenauswahl
			rcall	LCD_goto
			;------------------------------
alarm_hour:	sbiw	r24, 2			; EEPROM Adresse -2
			rcall	EERead			; Wert fr Stunden laden
			mov		r17, r18		; in r17 laden
			ldi		r16, 0			; Menu Abschnitt 0
			mov		input, r16
alarm_hour1:
			ldi		r16, 0x48		; Cursor auf Stunden
			rcall	LCD_goto	
			mov		r16, r17		; in r16 laden
			rcall 	sendBCD			; senden
			ldi		r16, 0x49		; Cursor auf Stundenauswahl
			rcall	LCD_goto
			rjmp	alarm_adjust1	; sprung zu alarm_adjust1
			;-----------------------------
alarm_minute:
			adiw	r24, 1			; EEPROM Adresse +1
			rcall	EERead			; Wert fr Minuten laden
			mov		r17, r18		; in r17 laden
			ldi		r16, 1			; Menu Abschnitt 1
			mov		input, r16
alarm_minute1:
			ldi		r16, 0x4B		; Cursor auf Minuten
			rcall	LCD_goto
			mov		r16, r17		; in r16 laden
			rcall	sendBCD			; senden
			ldi		r16, 0x4C		; Cursor auf Minutenauswahl
			rcall	LCD_goto
			rjmp	alarm_adjust1	; spring zu alarm_adjust1
			;-----------------------------
alarm_second:
			adiw	r24, 1			; EEPROM Adresse +1
			rcall	EERead			; Wert fr Sekunden laden
			mov		r17, r18		; in r17 laden
			ldi		r16, 2			; Menu Abschnitt 2
			mov		input, r16
alarm_second1:
			ldi		r16, 0x4E		; Cursor auf Sekunden
			rcall	LCD_goto
			mov		r16, r17		; in r16 laden
			rcall	sendBCD			; senden
			ldi		r16, 0x4F		; Cursor auf Sekunden auswahl
			rcall	LCD_goto
			;--------------------------------
alarm_adjust1:
			wdr						; wdr rcksetzen
			;--Tasten-------------
			cli						; Interrups sperren
 			mov 	r19, key_press  ; in r19 laden
			clr  	key_press   	; rcksetzen
			sei						; Interrups freigeben
			sbrc	r19,2			; Wenn Taster1
			rjmp	t1_alarm_a
			sbrc	r19,3
			rjmp	t2_alarm_a		; Wenn Taster2
			rjmp	alarm_adjust1	; von Vorne
;Sprungtabellen-------------------------------------------------------------------------------
menu_m_table:
			rjmp	m_alarm_adjust
			rjmp	m_time_adjust
			rjmp	m_back
menu_s_table:
			rjmp	mainloop0
			rjmp	alarm_adjust
			rjmp	time_adjust
alarm_time_table:
			rjmp	alarm_hour
			rjmp	alarm_minute
			rjmp	alarm_second
alarm_time_tableI:
			rjmp	alarm_hourI
			rjmp	alarm_minuteI
			rjmp	alarm_secondI
alarm_time_tableII:
			rjmp	alarm_hourII
			rjmp	alarm_minuteII
			rjmp	alarm_secondII
time_tableI:
			rjmp	time_yearI
			rjmp	time_monthI
			rjmp	time_dateI
			rjmp	time_dayI
			rjmp	time_hourI
			rjmp	time_minuteI
			rjmp	time_secondI
time_tableII:
			rjmp	time_yearII
			rjmp	time_monthII
			rjmp	time_dateII
			rjmp	time_dayII
			rjmp	time_hourII
			rjmp	time_minuteII
			rjmp	time_secondII
;----------------------------------------------------------------------------
onTC0:		cli							; Interrups sperren
			push	r16					; r16 sichern
			in  	save_sreg, SREG		; SREG sichern
get8key:	in  	iwr0, PINB			
  			com 	iwr0      			;low active
  			eor  	iwr0, key_state
  			and  	key_ct0, iwr0    	;reset counter
  			and  	key_ct1, iwr0
  			com  	key_ct0      		;count
  			eor  	key_ct1, key_ct0
  			and  	iwr0, key_ct0    	;input AND last counterstate
  			and  	iwr0, key_ct1
  			eor  	key_state, iwr0    ;toggle state
  			and  	iwr0, key_state    ;0-1-transition
  			or  	key_press, iwr0    ;store key press detect
			;------------------------------------
			ldi 	r16, 220			; TC0 zhlt bis 256 => 36 mit Vorteiler 1024 =>3686400/1024/36 = 100 Hz =0,01 s
  			out 	TCNT0, r16
			;---------------------
			;-------------------------
  			out  	SREG, save_sreg		; SREG wieder herstellen
			pop		r16					; r16 wieder herstellen	
  			sei							; Interrups freigeben
  			reti						; Rcksprung
;----------------------------------------------------------------------------
; Dateien laden
.include "lcd_control.s"
.include "utilities.s"
.include "lcd_out.s"
.include "twi_control.s"
.include "temp_control.s"
.include "rtc_control.s"
.include "alarm.s"
.include "input.s"
; Adressen fr EEPROM
eep_alarm:	.db 0x00, 0x00		; 0x000 - 0x002 fr Weckzeit EEPROM
; Text fr LCD
txt_date:	.db	"Datum",0				; Zeichenkette Datum
txt_time:	.db "Uhrzeit",0				; Zeichenkette Uhrzeit
txt_alarm:	.db "Weckerzeit",0,0		; Zeichenkette Weckerzeit
txt_back:	.db "Zur",0xF5,"ck",0,0		; Zeichenkette Zurck

