;-----------------------------------------------------------------------------
LCD_init:	push	r16
			sbi		DDRD,2			; LCD RS = OUT
			sbi		DDRD,3			; LCD E  = OUT
			sbi		DDRD,4			; LCD D4 = OUT
			sbi		DDRD,5			; LCD D5 = OUT
			sbi		DDRD,6			; LCD D6 = OUT
			sbi		DDRD,7			; LCD D7 = OUT
			cbi		PORTD,2			; LCD RS = Low
			; warte bis PowerUP
			ldi		r16,32			; Power-Up Wartezyklus min 30 ms	
			rcall	Wait_ms
			; sende Resetsequenz kompatibel zu HD44780 Industriestandard
			ldi		r16, 0b00110000	; Reset-Sequenz Teil 1
			out		PORTD, r16
			rcall	LCD_enable		; Enable-Impuls
			ldi		r16, 5
			rcall	wait_ms	
			ldi		r16, 0b00110000 ; Reset-Sequenz Teil 2
			out		PORTD, r16
			rcall	LCD_enable		; Enable-Impuls
			ldi		r16, 11
			rcall	wait_10us		; Wartezyklus bei RESET LCD min 100 s
			ldi		r16, 0b00110000	; Reset_Sequenz Teil 3
			out		PORTD, r16
			rcall	LCD_enable		; Enable-Impuls
			rcall	LCD_on
			; sende init 1
			ldi		r16, 0b00100000	; 4 Bit Modus aktivieren
			out		PORTD, r16
			rcall	LCD_enable		; Enable-Impuls
			ldi		r16, 5
			rcall	wait_10us		; Zeit zum Umschalten lassen
			ldi		r16, 0b00101000 
			rcall	LCD_cmd			; Function Set 4 Bit, 2 Zeilen, 5x7
			rcall	LCD_off
			rcall	LCD_clear
			ldi		r16, 0x06
			rcall	LCD_cmd			; Entry Mode Set, increase+shifted
			rcall	LCD_on
			pop 	r16
			ret
;-----------------------------------------------------------------------------
; sendet Daten oder Befehl an LCD
; PE: r16 = Daten oder Befehl
;---------------------------------------------------------------------------------
LCD_data:	push	r18
			ldi	 	r18,0b0000100	; RS = hi
			rjmp	LCD_out1
;-----------------------------------------------------------------------------
LCD_cmd:	push	r18
			ldi	 	r18,0b0000000	; RS = lo
LCD_out1:	push	r17
			mov	 	r17, r16
			swap	r17
			andi	r16, 0b11110000
			or	  	r16,r18
			andi	r17, 0b11110000
			or	  	r17,r18
			out	 	PORTD, r16
			rcall   LCD_enable
			out	 	PORTD, r17
			rcall   LCD_enable
			ldi		r16, 2
			rcall   wait_ms
			pop		r17
			pop		r18
			ret
;-----------------------------------------------------------------------------
LCD_enable:	sbi	 	PORTD, 3	  	; Enable high
			nop	 					; kurz warten
			nop
			nop
			cbi	 	PORTD, 3	  	; Enable wieder low
			ret
;-----------------------------------------------------------------------------
LCD_clear:  push	r16
			ldi	 	r16, 0b00000001	; Display lschen		
			rcall   LCD_cmd
			ldi		r16, 2
			rcall   wait_ms
			pop		r16
			ret
;-----------------------------------------------------------------------------
LCD_off:	push	r16
			ldi	 	r16, 0b00001000
			rcall   LCD_cmd
			ldi		r16, 5
			rcall   wait_10us
			pop		r16
			ret
;-----------------------------------------------------------------------------
LCD_on:		push	r16
			ldi		r16, 0b00001110
			rcall   LCD_cmd
			ldi		r16, 5
			rcall   wait_10us
			pop		r16
			ret
;-----------------------------------------------------------------------------
LCD_home:   push	r16
			ldi		r16, 0b00000010		; Display Cursor HOME
			rcall   LCD_cmd
			ldi		r16, 2
			rcall   wait_ms
			pop		r16
			ret
;-----------------------------------------------------------------------------
LCD_line1:  push	r16
			ldi	 	r16, 0b10000000	  ; DRAM auf Adresse 0x00
			rcall   LCD_cmd
			ldi		r16, 5
			rcall   wait_10us
			pop		r16
			ret
;-----------------------------------------------------------------------------
LCD_line2:  push	r16
			ldi		r16, 0b11000000	  ; DRAM auf Adresse 0x40
			rcall   LCD_cmd
			ldi		r16, 5
			rcall   wait_10us
			pop		r16
			ret
;-----------------------------------------------------------------------------
; Goto r16 = Adresse (Zeile 1 = 0x00..0x0F, Zeile 2 = 0x40..0x4F)
LCD_goto:	ori		r16, 0b10000000	  ; Goto DRAM auf Adresse r16
			rcall   LCD_cmd
			ldi		r16, 5
			rcall   wait_10us
			ret
;-----------------------------------------------------------------------------
; sendet ein BCD codiertes Byte als zwei ASCII-Ziffern -> LCD
; PE: r16 
; PA: r16 unverndert
;---------------------------------------------------------------
sendBCD:	push	r16			; r16 sichern
			swap	r16			; Halfbytes tauschen
			andi	r16,0x0f	; nur unter 4 Bit
			ori		r16,0x30	; 30 addieren fr ASCII
			rcall	LCD_data	; 1. Halfbyte senden
			pop		r16			; r16 wiederherstellen
			push	r16			; r16 sichern
			andi	r16,0x0f	; nur unter 4 Bit
			ori		r16,0x30	; 30 addieren fr ASCII
			rcall	LCD_data	; 2. Halfbyte senden
			pop		r16			; r16 wiederherstellen
			ret					; Rcksprung
;-----------------------------------------------------------------------------
